==> Smdn.Net.Imap4.Client-1.01.tar.bz2: Smdn.Net.Imap4.Client-1.01/Smdn.Net.Imap4/Smdn.Net.Imap4/ImapNamespaceDesc.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Formats;

namespace Smdn.Net.Imap4 {
  public sealed class ImapNamespaceDesc : ICloneable {
    public string Prefix {
      get; private set;
    }

    public string HierarchyDelimiter {
      get; private set;
    }

#if NET_4_5
    public IReadOnlyDictionary<string, string[]> Extensions {
#else
    public IDictionary<string, string[]> Extensions {
#endif
      get; private set;
    }

    /*
     * RFC 5255 - Internet Message Access Protocol Internationalization
     * http://tools.ietf.org/html/rfc5255
     * 3.4. TRANSLATION Extension to the NAMESPACE Response
     */
    public string TranslatedPrefix {
      get
      {
        string[] translatedPrefix;

        if (Extensions.TryGetValue("TRANSLATION", out translatedPrefix))
          return ModifiedUTF7.Decode(translatedPrefix[0]);
        else
          return Prefix;
      }
    }

#if NET_4_5
    private static readonly IReadOnlyDictionary<string, string[]> readOnlyEmptyDictionary = new Dictionary<string, string[]>(1);
#else
    private static readonly IDictionary<string, string[]> readOnlyEmptyDictionary = (new Dictionary<string, string[]>(1)).AsReadOnly();
#endif

    public ImapNamespaceDesc(string prefix)
      : this(prefix, null, readOnlyEmptyDictionary, false)
    {
    }

    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter)
      : this(prefix, hierarchyDelimiter, readOnlyEmptyDictionary, false)
    {
    }

#if NET_4_5
    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter, IReadOnlyDictionary<string, string[]> extensions)
#else
    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter, IDictionary<string, string[]> extensions)
#endif
      : this(prefix, hierarchyDelimiter, extensions, true)
    {
    }

#if NET_4_5
    private ImapNamespaceDesc(string prefix, string hierarchyDelimiter, IReadOnlyDictionary<string, string[]> extensions, bool checkExtensions)
#else
    private ImapNamespaceDesc(string prefix, string hierarchyDelimiter, IDictionary<string, string[]> extensions, bool checkExtensions)
#endif
    {
      if (prefix == null)
        throw new ArgumentNullException("prefix");

      this.Prefix = prefix;
      this.HierarchyDelimiter = hierarchyDelimiter;

      if (checkExtensions) {
        if (extensions == null)
          throw new ArgumentNullException("extensions");

#if NET_4_5
        this.Extensions = extensions;
#else
        if (extensions.IsReadOnly)
          this.Extensions = extensions.AsReadOnly();
        else
          this.Extensions = extensions;
#endif
      }
      else {
        this.Extensions = extensions;
      }
    }

    object ICloneable.Clone()
    {
      return this.Clone();
    }

    public ImapNamespaceDesc Clone()
    {
      var cloned = (ImapNamespaceDesc)MemberwiseClone();

      if (Extensions == readOnlyEmptyDictionary) {
        cloned.Extensions = readOnlyEmptyDictionary;
      }
      else {
        var extensions = new Dictionary<string, string[]>(Extensions.Count, StringComparer.OrdinalIgnoreCase);

        foreach (var pair in Extensions) {
          extensions.Add(pair.Key, (string[])pair.Value.Clone());
        }

#if NET_4_5
        cloned.Extensions = extensions;
#else
        cloned.Extensions = extensions.AsReadOnly();
#endif
      }

      return cloned;
    }

    public override string ToString()
    {
      return string.Format("{{Prefix={0}, HierarchyDelimiter={1}, Extensions={2}}}",
                           Prefix,
                           HierarchyDelimiter,
                           Extensions);
    }
  }
}

==> Smdn.Net.Imap4.Client-1.09.tar.bz2: Smdn.Net.Imap4.Client-1.09/Smdn.Net.Imap4/Smdn.Net.Imap4/ImapNamespaceDesc.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Formats;

namespace Smdn.Net.Imap4 {
  public sealed class ImapNamespaceDesc : ICloneable {
    public string Prefix {
      get; private set;
    }

    public string HierarchyDelimiter {
      get; private set;
    }

#if NET_4_5
    public IReadOnlyDictionary<string, string[]>
#else
    public IDictionary<string, string[]>
#endif
    Extensions {
      get; private set;
    }

    /*
     * RFC 5255 - Internet Message Access Protocol Internationalization
     * http://tools.ietf.org/html/rfc5255
     * 3.4. TRANSLATION Extension to the NAMESPACE Response
     */
    public string TranslatedPrefix {
      get
      {
        string[] translatedPrefix;

        if (Extensions.TryGetValue("TRANSLATION", out translatedPrefix))
          return ModifiedUTF7.Decode(translatedPrefix[0]);
        else
          return Prefix;
      }
    }

    public ImapNamespaceDesc(string prefix)
      : this(prefix, null, null)
    {
    }

    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter)
      : this(prefix, hierarchyDelimiter, null)
    {
    }

#if NET_4_5
    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter, IReadOnlyDictionary<string, string[]> extensions)
#else
    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter, IDictionary<string, string[]> extensions)
#endif
    {
      if (prefix == null)
        throw new ArgumentNullException("prefix");

      this.Prefix = prefix;
      this.HierarchyDelimiter = hierarchyDelimiter;
      this.Extensions = CollectionUtils.ToNonNullReadOnly(extensions);
    }

    object ICloneable.Clone()
    {
      return this.Clone();
    }

    public ImapNamespaceDesc Clone()
    {
      var cloned = (ImapNamespaceDesc)MemberwiseClone();

      var extensions = new Dictionary<string, string[]>(Extensions.Count, StringComparer.OrdinalIgnoreCase);

      foreach (var pair in Extensions) {
        extensions.Add(pair.Key, (string[])pair.Value.Clone());
      }

#if NET_4_5
      cloned.Extensions = extensions;
#else
      cloned.Extensions = extensions.AsReadOnly();
#endif

      return cloned;
    }

    public override string ToString()
    {
      return string.Format("{{Prefix={0}, HierarchyDelimiter={1}, Extensions={2}}}",
                           Prefix,
                           HierarchyDelimiter,
                           CollectionUtils.Join(", ", Extensions));
    }
  }
}
